//using CompiledModelDemo.TheCompiledStuff;
using System;
using System.Drawing;
using System.Linq;
using System.Linq.Expressions;
using EFCore6Demos;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Storage.ValueConversion;
using static System.Environment;

public class PeopleContext : DbContext {
  public DbSet<Person> People { get; set; }

  protected override void OnConfiguring (DbContextOptionsBuilder optionsBuilder) {
    _ = optionsBuilder.LogTo (Console.WriteLine, Microsoft.Extensions.Logging.LogLevel.Information)
      .UseSqlServer ("Server=localhost,1604;Database=TemporalTest;User ID=sa;Password=P@ssword1;MultipleActiveResultSets=true;Connect Timeout=30;");
  }
  protected override void OnModelCreating (ModelBuilder modelBuilder) {
    ////The old cumbersome way of expressing bulk conversions, replaced in ConfigureConventions method
    // foreach (var entity in modelBuilder.Model.GetEntityTypes ()) 
    // {
    //   foreach (var prop in entity.GetProperties ().Where (p => p.ClrType == typeof (string)))
    //   {
    //     prop.SetColumnType ("varchar(100)");
    //   }
    // }
    // }
    //}
    
    ////Former way to implement custom converter e.g. color, one property at a time
    ////replaced with bulk convention in ConfigureConventions method
    // modelBuilder.Entity<Address>().Property(ad=>ad.StructureColor)
    // .HasConversion(c=>c.ToString(),s=>Color.FromName(s));
       
    ////Pattern for overriding a bulk convention (defined in ConfigureConventions method) for a single property
    //  modelBuilder.Entity<Address>().Property(ad=>ad.StructureColor)
    //     .HasConversion((ValueConverter?)null);
  }
  protected override void ConfigureConventions (ModelConfigurationBuilder configurationBuilder) {

    configurationBuilder.Properties<string>().HaveColumnType("nvarchar(100)");
    configurationBuilder.Properties<AddressType> ().HaveConversion<string> ();
    configurationBuilder.Properties<Color>().HaveConversion(typeof(ColorToStringConverter));
   }
}

public class ColorToStringConverter : ValueConverter<Color, string> {
  public ColorToStringConverter () : base (ColorString, ColorStruct) { }
  private static Expression<Func<Color, string>> ColorString = v => new String (v.Name);
  private static Expression<Func<string, Color>> ColorStruct = v => Color.FromName (v);

}